// simple example of using the tree glyph.  
// inspired by Doug Youngs tree widget.
// Dave Sternlicht - MIT X Consortium

#include <InterViews/background.h>
#include <InterViews/color.h>
#include <InterViews/font.h>
#include <InterViews/label.h>
#include <InterViews/layout.h>
#include <InterViews/session.h>
#include <InterViews/window.h>
#include "tree.h"

// 72 dots per inch.
#define DPI 72.0
#define TREE_WIDTH DPI * 3.0
#define TREE_HEIGHT TREE_WIDTH

struct Node { int key; Node* left; Node* right; };

class App {
public:
    int run(int, char**);
    Node* insert_node(int, Node*);
    Node* make_node(int);
    void show_tree(Glyph*, Node*);
private:
    ApplicationWindow* main_;
    LayoutKit* layout_;
    Session* session_;
    Tree* tree_;
    const Color* white_;
    const Color* black_;
    Font* font_;
};

int main(int argc, char** argv) {
    App* a = new App; return a->run(argc, argv);
}

int App::run(int argc, char** argv) {
    Node* head = nil;
    int digit;

    session_ = new Session("Sort", argc, argv);
    white_ = Color::lookup(session_->default_display(), "white");
    black_ = Color::lookup(session_->default_display(), "black");
    font_ = new Font("fixed");
    layout_ = LayoutKit::instance();
    LayoutKit& layout = *layout_;
    main_ = new ApplicationWindow(
        new Background(
            layout.fixed_span(
                tree_ = new Tree(session_), TREE_WIDTH, TREE_HEIGHT
            ),
            white_
        )
    );
    // create a binary sort tree from data read from stdin.
    while (scanf("%d", &digit) != EOF)
        head = insert_node(digit, head);
    // create the widgets representing the tree.
    show_tree(nil, head);
    tree_->force_layout();
    return session_->run_window(main_);    
}

Node* App::insert_node(int key, Node* head) { 
    Node* prev; Node* ptr = head;
    // if the tree doesn't exist, just create and return a new node.
    if (!head)
        return (make_node(key));

    // otherwise find a leaf node, always following the left branches if the
    // key is less than the value in each node, and the right branch otherwise.
    while (ptr != nil) {
        prev = ptr;
        ptr = (key < ptr->key) ? ptr->left : ptr->right;
    }

    // make a new node and attach it to the appropriate branch.
    if (key < prev->key)
        prev->left = make_node(key);
    else
        prev->right = make_node(key);
    return (head);
}

Node* App::make_node(int key) { 
    Node* ptr = new Node;
    ptr->key = key;
    ptr->left = ptr->right = nil;
    return ptr;
}

void App::show_tree(Glyph* parent, Node* branch) { 
    Label* l;
    char* name = new char[100];
    int n=0;
    
    // if we've hit a leaf, return.
    if (!branch) return;

    // create a glyph for the node, specifying the given super_node constraint.
    sprintf(name, "%d", branch->key);
    
    tree_->insert_node(parent, l = new Label(name, font_, black_));
    show_tree(l, branch->left);
    show_tree(l, branch->right);
} 
